@@ -28,6 +28,87 @@ concentration_basic <- function(dep.var = 1, state, cache, i, j, edge, update, g
 }
 
 
+#' concentration_AC
+#' 
+#' Is there a bandwagon effect in mobility, i.e. do mobile individuals move to locations 
+#' that are the destination of many others from their origin? The functional form of this 
+#' statistic assumes that there are decreasing additional returns to more others on the 
+#' same mobility path. For example, the probability to choose a mobility path that already 
+#' contains 20 other individuals is hardly different from a path with 25 other individuals; 
+#' however, there is a substantial difference in the comparison of paths with 2 or 7 other 
+#' individuals.
+#' 
+#' @param dep.var 
+#' @param state 
+#' @param cache 
+#' @param i 
+#' @param j 
+#' @param edge 
+#' @param update 
+#' @param getTargetContribution 
+#' @param alpha 
+#'
+#' @return Returns the change statistic or target statistic of the effect for 
+#' internal use by the estimation algorithm.
+#' @keywords internal
+concentration_AC <- function(dep.var = 1, state, cache, i, j, edge, update, 
+                             getTargetContribution = FALSE, alpha = 2){
+  if(alpha < 1) stop("alpha parameter in concentration_AC function must be 1 or larger")
+  
+  if(i == j) return(0)
+  
+  if(getTargetContribution){
+    
+    g_cum <- function(y, a){
+      contr <- 0
+      
+      #for(k in 0:y){
+      #  contr <- contr + (y-k) * exp(-log(a)*k)
+      #}
+      #contr - y
+      
+      if(y>1){
+        for(k in 1:(y-1)){
+          contr <- contr + (1 - (1-1/a)^(k))
+        }
+      }
+      return(contr)
+    }
+    
+    nResources <- cache[[dep.var]]$valuedNetwork[i, j]
+    
+    return(g_cum(y = nResources, a = alpha))
+  }
+  
+  ### calculate change statistic
+  
+  g_mar <- function(y, a){
+    contr <- 0
+    
+    #for(k in 0:y){
+    #  contr <- contr + exp(-log(a)*k)
+    #}
+    #contr - 1
+    
+    if(y>0) {
+      contr <-  (1 - (1-1/a)^(y)) 
+    } else {
+      contr <- 0
+    }
+    return(contr)
+  }
+  
+  tie_val <- cache[[dep.var]]$valuedNetwork[i, j]
+
+  if(update < 0){
+    return(update * g_mar(y = (tie_val + update), a = alpha))
+  }
+  if(update > 0){
+    return(update * g_mar(y = tie_val, a = alpha))
+  }
+}
+
+
 #' concentration_GW
 #' 
 #' Is there a bandwagon effect in mobility, i.e. do mobile individuals move to locations 
@@ -83,7 +164,7 @@ concentration_GW <- function(dep.var = 1, state, cache, i, j, edge, update,
   }
   
   tie_val <- cache[[dep.var]]$valuedNetwork[i, j]
-
+  
   if(update < 0){
     return(update * g_mar(y = (tie_val + update), a = alpha))
   }
@@ -93,6 +174,95 @@ concentration_GW <- function(dep.var = 1, state, cache, i, j, edge, update,
 }
 
 
+#' concentration_AC_resource_covar_bin
+#' 
+#' Is there a bandwaggon effect in mobility, akin to the concentration_AC
+#' effect, but where people of group 1 only consider others of group 1
+#' in their decision to move?
+#' 
+#' @param dep.var 
+#' @param state 
+#' @param resource.attribute.index 
+#' @param cache 
+#' @param i 
+#' @param j 
+#' @param edge 
+#' @param update 
+#' @param getTargetContribution 
+#' @param alpha 
+#'
+#' @return Returns the change statistic or target statistic of the effect for 
+#' internal use by the estimation algorithm.
+#' @keywords internal
+concentration_AC_resource_covar_bin <- function(dep.var = 1, 
+                                                state,
+                                                resource.attribute.index,
+                                                cache, i, j, 
+                                                edge,
+                                                update, 
+                                                getTargetContribution = FALSE, 
+                                                alpha = 2){
+  if(alpha < 1) stop("alpha parameter in concentration_AC_resource_covar_bin function must be 1 or larger")
+  
+  if(i == j) return(0)
+  
+  if(getTargetContribution){
+    
+    g_cum <- function(y, a){
+      contr <- 0
+      
+      #for(k in 0:y){
+      #  contr <- contr + (y-k) * exp(-log(a)*k)
+      #}
+      #contr - y
+      
+      if(y>1){
+        for(k in 1:(y-1)){
+          contr <- contr + (1 - (1-1/a)^(k))
+        }
+      }
+      return(contr)
+    }
+    
+    nResources <- cache[[dep.var]]$resourceNetworks[[resource.attribute.index]][i, j]
+    
+    return(g_cum(y = nResources, a = alpha))
+  }
+  
+  ### calculate change statistic
+  
+  # if resource is of type 0, it cannot contribute
+  
+  if(state[[resource.attribute.index]]$data[edge] == 0){
+    return(0)
+  }
+  
+  g_mar <- function(y, a){
+    contr <- 0
+    
+    #for(k in 0:y){
+    #  contr <- contr + exp(-log(a)*k)
+    #}
+    #contr - 1
+    
+    if(y>0) {
+      contr <-  (1 - (1-1/a)^(y)) 
+    } else {
+      contr <- 0
+    }
+    return(contr)
+  }
+  
+  tie_val <- cache[[dep.var]]$resourceNetworks[[resource.attribute.index]][i, j]
+  
+  if(update < 0){
+    return(update * g_mar(y = (tie_val + update), a = alpha))
+  }
+  if(update > 0){
+    return(update * g_mar(y = tie_val, a = alpha))
+  }
+}
+
 #' concentration_GW_resource_covar_bin
 #' 
 #' Is there a bandwaggon effect in mobility, akin to the concentration_GW
@@ -121,7 +291,7 @@ concentration_GW_resource_covar_bin <- function(dep.var = 1,
                                                 update, 
                                                 getTargetContribution = FALSE, 
                                                 alpha = 2){
-  if(alpha < 1) stop("alpha parameter in concentration_GW function must be 1 or larger")
+  if(alpha < 1) stop("alpha parameter in concentration_GW_resource_covar_bin function must be 1 or larger")
   
   if(i == j) return(0)
   
@@ -167,6 +337,91 @@ concentration_GW_resource_covar_bin <- function(dep.var = 1,
 }
 
 
+#' concentration_AC_dyad_covar
+#' 
+#' Are bandwagon effects (concentration) particularly prevalent between locations 
+#' that share characteristics as encoded in a binary dyadic covariate? E.g., do 
+#' workers follow the moves of other workers mainly in case they go to organisations 
+#' in the same region?
+#' 
+#' @param dep.var 
+#' @param attribute.index 
+#' @param state 
+#' @param cache 
+#' @param i 
+#' @param j 
+#' @param edge 
+#' @param update 
+#' @param getTargetContribution 
+#' @param alpha 
+#'
+#' @return Returns the change statistic or target statistic of the effect for 
+#' internal use by the estimation algorithm.
+#' @keywords internal
+concentration_AC_dyad_covar <- function(dep.var = 1, attribute.index, state, cache, i, j, edge, update, 
+                                            getTargetContribution = FALSE, alpha = 2){
+  if(alpha < 1) stop("alpha parameter in concentration_AC_dyad_covar function must be 1 or larger")
+  # if(!all(state[[attribute.index]]$data == t(state[[attribute.index]]$data))) stop("attribute.index in concentration_AC_dyad_covar_bin function must be symmetric")
+  # if(!all(state[[attribute.index]]$data %in% c(0,1))) stop("all values of attribute.index in concentration_AC_dyad_covar_bin function must be 0 or 1")
+  if(dim(state[[attribute.index]]$data)[1] != dim(state[[attribute.index]]$data)[2]) stop("attribute.index in concentration_AC_dyad_covar_bin function must be a square matrix")
+  if(length(dim(state[[attribute.index]]$data)) != 2) stop("attribute.index in concentration_AC_dyad_covar_bin function must be a square matrix")
+  
+  if(i == j) return(0)
+  
+  if(getTargetContribution){
+    
+    g_cum <- function(y, a){
+      contr <- 0
+      
+      #for(k in 0:y){
+      #  contr <- contr + (y-k) * exp(-log(a)*k)
+      #}
+      #contr - y
+      
+      if(y>1){
+        for(k in 1:(y-1)){
+          contr <- contr + (1 - (1-1/a)^(k))
+        }
+      }
+      return(contr)
+    }
+    
+    nResources <- cache[[dep.var]]$valuedNetwork[i, j] 
+    
+    return(g_cum(y = nResources, a = alpha) * state[[attribute.index]]$data[i, j])
+  }
+  
+  ### calculate change statistic
+  
+  if(state[[attribute.index]]$data[i, j] == 0) return(0)
+  
+  g_mar <- function(y, a){
+    contr <- 0
+    
+    #for(k in 0:y){
+    #  contr <- contr + exp(-log(a)*k)
+    #}
+    #contr - 1
+    
+    if(y>0) {
+      contr <-  (1 - (1-1/a)^(y)) 
+    } else {
+      contr <- 0
+    }
+    return(contr)
+  }
+  
+  tie_val <- cache[[dep.var]]$valuedNetwork[i, j] 
+  
+  if(update < 0){
+    return(update * g_mar(y = (tie_val + update), a = alpha) * state[[attribute.index]]$data[i, j])
+  }
+  if(update > 0){
+    return(update * g_mar(y = tie_val, a = alpha) * state[[attribute.index]]$data[i, j])
+  }
+}
+
+
 #' concentration_GW_dyad_covar
 #' 
 #' Are bandwagon effects (concentration) particularly prevalent between locations 
@@ -189,7 +444,7 @@ concentration_GW_resource_covar_bin <- function(dep.var = 1,
 #' internal use by the estimation algorithm.
 #' @keywords internal
 concentration_GW_dyad_covar <- function(dep.var = 1, attribute.index, state, cache, i, j, edge, update, 
-                                            getTargetContribution = FALSE, alpha = 2){
+                                        getTargetContribution = FALSE, alpha = 2){
   if(alpha < 1) stop("alpha parameter in concentration_GW function must be 1 or larger")
   # if(!all(state[[attribute.index]]$data == t(state[[attribute.index]]$data))) stop("attribute.index in concentration_GW_dyad_covar_bin function must be symmetric")
   # if(!all(state[[attribute.index]]$data %in% c(0,1))) stop("all values of attribute.index in concentration_GW_dyad_covar_bin function must be 0 or 1")
@@ -419,6 +674,7 @@ concentration_basic_squared <- function(dep.var = 1, state, cache, i, j, edge, u
 }
 
 
+
 #' concentration_norm
 #' 
 #' Is there a bandwagon effect in mobility, i.e. do mobile individuals move to 
@@ -491,4 +747,33 @@ concentration_norm_squared <- function(dep.var = 1, state, cache, i, j, edge, up
     }
     return(cont)
   }
-}
\ No newline at end of file
+}
+
+
+#' concentration_basic_cube
+#' 
+#' Is there a bandwagon effect in mobility, i.e. do mobile individuals move to 
+#' locations that are the destination of many others from their origin?
+#' 
+#' @param dep.var 
+#' @param state 
+#' @param cache 
+#' @param i 
+#' @param j 
+#' @param edge 
+#' @param update 
+#' @param getTargetContribution 
+#'
+#' @return Returns the change statistic or target statistic of the effect for 
+#' internal use by the estimation algorithm.
+#' @keywords internal
+concentration_basic_cube <- function(dep.var = 1, state, cache, i, j, edge, update, getTargetContribution = FALSE){
+  if(getTargetContribution){
+    return( (cache[[dep.var]]$valuedNetwork[i, j])^3 )
+  } else {
+    v <- (cache[[dep.var]]$valuedNetwork[i, j] + update)^3 - 
+      (cache[[dep.var]]$valuedNetwork[i, j])^3
+    return(v)
+  }
+}
+
